# import standard python libraries
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import colors

import pandas as pd
import os, subprocess

# import core packages
import warnings
warnings.filterwarnings("ignore")
from itertools import combinations

# import semi-core packages

plt.style.use('seaborn-poster')
import numpy as np


# import open2c libraries
import bioframe

# Import python package for working with cooler files and tools for analysis
import cooler
import cooltools
import cooltools.lib.plotting

from packaging import version
if version.parse(cooltools.__version__) < version.parse('0.5.4'):
    raise AssertionError("tutorials rely on cooltools version 0.5.4 or higher,"+
                         "please check your cooltools version and update to the latest")
                         
import cooltools

def Ps(clr):
    
    # select only those chromosomes available in cooler
    arms = GRCm38_arms[GRCm38_arms.chrom.isin(clr.chromnames)].reset_index(drop=True)
    print('df')
    
    
    # cvd == contacts-vs-distance
    cvd = cooltools.expected_cis(
        clr=clr,
        view_df=arms,
        smooth=False,
        aggregate_smoothed=False,
        nproc=4 #if you do not have multiple cores available, set to 1
        )
    print('cvd')
    cvd['s_bp'] = cvd['dist']* resolution
    return arms, cvd

def Ps_smooth(clr, GRCm38_arms):
    cvd_smooth_agg = cooltools.expected_cis(
        clr=clr,
        view_df=GRCm38_arms,
        smooth=True,
        aggregate_smoothed=True,
        nproc=4
    )
    
    cvd_smooth_agg['s_bp'] = cvd_smooth_agg['dist']* resolution
    cvd_smooth_agg['balanced.avg.smoothed'].loc[cvd_smooth_agg['dist'] < 2] = np.nan
    
    return cvd_smooth_agg

def Ps_smooth_agg(cvd_smooth_agg):
    cvd_smooth_agg['s_bp'] = cvd_smooth_agg['dist']* resolution
    cvd_smooth_agg['balanced.avg.smoothed.agg'].loc[cvd_smooth_agg['dist'] < 2] = np.nan
    return cvd_smooth_agg

def Ps_curve(cvd_smooth_agg):
    # Just take a single value for each genomic separation
    cvd_merged = cvd_smooth_agg.drop_duplicates(subset=['dist'])[['s_bp', 'balanced.avg.smoothed.agg']]
    
    #norm on s = 100kb
    val = cvd_merged.loc[cvd_merged.s_bp==100000, 'balanced.avg.smoothed.agg'].values[0]
    cvd_merged['balanced.avg.smoothed.agg.norm'] = cvd_merged['balanced.avg.smoothed.agg']/val
        
    # Calculate derivative in log-log space
    # der = np.gradient(np.log(cvd_merged['balanced.avg.smoothed.agg']),
                  # np.log(cvd_merged['s_bp']))
    der = np.gradient(np.log(cvd_merged['balanced.avg.smoothed.agg.norm']),
                  np.log(cvd_merged['s_bp']))
    return cvd_merged, der


resolution = 10000 # note this might be slightly slow on a laptop

samples = [
    # 'RA1', 'RA2',
    # 'RC1',  'RC2',
    # 'dMR231-A', 'dMR70-A',
    # 'dMR231-C', 'dMR70-C',
    # 'CapH_aux_r1', 'CapH_aux_r2',
    # 'CapH_C_r1', 'CapH_C_r2',
    'CapH2_aux_r1', 'CapH2_aux_r2',
    'CapH2_C_r1', 'CapH2_C_r2'
    ]

color = [
    # 'brick red', 'brick red',
    # 'light salmon', 'light salmon',
    'dark blue', 'dark blue',
    'light blue grey', 'light blue grey',
    'jungle green', 'jungle green',
    'light aquamarine', 'light aquamarine',
    'darkish purple', 'darkish purple',
    'light purple', 'light purple'
    ]

path = '/storage3/svetlana/degron/hic'

GRCm38_chromsizes = bioframe.fetch_chromsizes('mm10')
GRCm38_cens = bioframe.fetch_centromeres('mm10')
# create a view with chromosome arms using chromosome sizes and definition of centromeres
GRCm38_arms = bioframe.make_chromarms(GRCm38_chromsizes,  GRCm38_cens)

f, axs = plt.subplots(
    figsize=(8,12),
    nrows=2,
    gridspec_kw={'height_ratios':[6,2]},
    sharex=True,
    # sharey=True
    )
   
for i in range(len(samples)):
    clr = cooler.Cooler(f'{path}/10kb/{samples[i]}_10_kb.cool')
    GRCm38_arms_mod, cvd = Ps(clr)
    #delete chrY
    GRCm38_arms_mod = GRCm38_arms_mod.loc[GRCm38_arms_mod.chrom != 'chrY']
    cvd_smooth_mod = Ps_smooth(clr, GRCm38_arms_mod)
    cvd_smooth_agg_mod = Ps_smooth_agg(cvd_smooth_mod)
    # cvd_merged_mod, der_mod = Ps_smooth_agg(cvd_smooth_agg_mod)
    cvd_merged_mod, der_mod = Ps_curve(cvd_smooth_agg_mod)
    
    f.suptitle("CapH2 aux vs CapH2 ctr", fontsize='xx-large')
    ax = axs[0]
    ax.plot(
        cvd_merged_mod['s_bp'],
        cvd_merged_mod['balanced.avg.smoothed.agg.norm'],
        '-',
        markersize=5,
        label = f'{samples[i]}',
        c=f'xkcd:{color[i]}'
        )
    ax.set_yscale('log')    
    ax.set(
        ylabel='IC contact frequency',
        # xlim=(1e3,1e8)
    )
    ax.set_aspect(1.0)
    ax.grid(lw=0.5)
    ax.legend()
        
        
    ax = axs[1]
    ax.semilogx(
        cvd_merged_mod['s_bp'],
        der_mod,
        # alpha=0.5,
        label = f'{samples[i]}',
        c=f'xkcd:{color[i]}',
        # lw=1
    )
    
    ax.set(
        xlabel='separation, bp',
        ylabel='slope',
        ylim=(-1.75,-0.15))
    
    ax.grid(lw=0.5)
    # ax.legend()
    
plt.tight_layout()    
plt.show()
plt.savefig(f'{path}/CapH2.noY.P_s_curve.png')    
print('end')




